package api

import (
	"archive/zip"
	"fmt"
	"io"
	"io/fs"
	"log"
	"net/http"
	"os"
	"path"
	"path/filepath"
	"strconv"
	"strings"
	"time"
)

func IsExist(path string) bool {
	_, err := os.Stat(path)
	if err != nil {
		if os.IsExist(err) {
			return true
		}
		if os.IsNotExist(err) {
			return false
		}
		fmt.Println(err)
		return false
	}
	return true
}

// WriteCounter counts the number of bytes written to it. It implements to the io.Writer
// interface and we can pass this into io.TeeReader() which will report progress on each
// write cycle.

type DownPercentCb func(w *WriteCounter)
type DownStartCb func(w *WriteCounter)
type DownFailCb func(err string)
type DownSuccCb func()

type WriteCounter struct {
	Total      uint64
	Curr       uint64
	OnProgress DownPercentCb
	OnStart    DownStartCb
	Etag       string
	TargetFile string
}

func (wc *WriteCounter) Write(p []byte) (int, error) {
	n := len(p)
	wc.Curr += uint64(n)
	if wc.OnProgress != nil {
		wc.OnProgress(wc)
	}
	return n, nil
}

// 超时时间20分钟
func DownFile(url string, targetFile string, timeoutSecond time.Duration, onStart DownStartCb, onProgress DownPercentCb, onFaild DownFailCb, onSucc DownSuccCb) error {
	client := http.DefaultClient
	client.Timeout = timeoutSecond

	dir := filepath.Dir(targetFile)
	os.MkdirAll(dir, os.ModePerm)

	if strings.HasPrefix(url, "//") {
		url = "http:" + url
	}
	fmt.Println("get url->", url)
	reps, err := client.Get(url)
	if err != nil {
		onFaild(err.Error())
		fmt.Println(err.Error())
		return err
	}
	fmt.Println("get url back", url)
	if reps.StatusCode != http.StatusOK {
		onFaild("url get 错误")
		return fmt.Errorf("url get 错误")
	}
	length := reps.Header.Get("Content-Length")
	etag := reps.Header.Get("Etag")
	counter := &WriteCounter{OnProgress: onProgress, OnStart: onStart}

	counter.Etag = etag
	counter.TargetFile = targetFile

	if counter.OnStart != nil {
		counter.OnStart(counter)
	}
	
	file, err := os.Create(counter.TargetFile)
	if err != nil {
		onFaild(err.Error())
		fmt.Println(err.Error())
		return err
	}
	defer file.Close() //关闭文件
	//获取下载文件的大小
	size, _ := strconv.ParseInt(length, 10, 64)

	counter.Total = uint64(size)
	_, err = io.Copy(file, io.TeeReader(reps.Body, counter))
	if err != nil {
		onFaild(err.Error())
	} else {
		onSucc()
	}
	return err
}

func Unzip(zipPath, dstDir string, cb func(per1000 int)) error {
	// open zip file
	reader, err := zip.OpenReader(zipPath)
	if err != nil {
		return err
	}
	defer reader.Close()

	total := len(reader.File)
	for index, file := range reader.File {
		per1000 := int(float32(index+1) / float32(total) * 1000)
		log.Println("unzip=>", file.Name, index, "/", len(reader.File))
		if cb != nil {
			cb(per1000)
		}
		if err := unzipFile(file, dstDir); err != nil {
			return err
		}
	}
	return nil
}

func unzipFile(file *zip.File, dstDir string) error {
	// create the directory of file
	filePath := path.Join(dstDir, file.Name)
	if file.FileInfo().IsDir() {
		if err := os.MkdirAll(filePath, os.ModePerm); err != nil {
			return err
		}
		return nil
	}
	if err := os.MkdirAll(filepath.Dir(filePath), os.ModePerm); err != nil {
		return err
	}

	// open the file
	rc, err := file.Open()
	if err != nil {
		return err
	}
	defer rc.Close()

	// create the file
	w, err := os.Create(filePath)
	if err != nil {
		return err
	}
	defer w.Close()

	// save the decompressed file content
	_, err = io.Copy(w, rc)
	return err
}

// Zip compresses the specified files or dirs to zip archive.
// If a path is a dir don't need to specify the trailing path separator.
// For example calling Zip("archive.zip", "dir", "csv/baz.csv") will get archive.zip and the content of which is
// baz.csv
// dir
// ├── bar.txt
// └── foo.txt
// Note that if a file is a symbolic link it will be skipped.
func Zip(zipPath string, paths ...string) error {
	// Create zip file and it's parent dir.
	if err := os.MkdirAll(filepath.Dir(zipPath), os.ModePerm); err != nil {
		return err
	}
	archive, err := os.Create(zipPath)
	if err != nil {
		return err
	}
	defer archive.Close()

	// New zip writer.
	zipWriter := zip.NewWriter(archive)
	defer zipWriter.Close()

	// Traverse the file or directory.
	for _, rootPath := range paths {
		// Remove the trailing path separator if path is a directory.
		rootPath = strings.TrimSuffix(rootPath, string(os.PathSeparator))

		// Visit all the files or directories in the tree.
		err = filepath.Walk(rootPath, walkFunc(rootPath, zipWriter))
		if err != nil {
			return err
		}
	}
	return nil
}

func walkFunc(rootPath string, zipWriter *zip.Writer) filepath.WalkFunc {
	return func(path string, info fs.FileInfo, err error) error {
		if err != nil {
			return err
		}

		// If a file is a symbolic link it will be skipped.
		if info.Mode()&os.ModeSymlink != 0 {
			return nil
		}

		// Create a local file header.
		header, err := zip.FileInfoHeader(info)
		if err != nil {
			return err
		}

		// Set compression method.
		header.Method = zip.Deflate

		// Set relative path of a file as the header name.
		header.Name, err = filepath.Rel(filepath.Dir(rootPath), path)
		if err != nil {
			return err
		}
		if info.IsDir() {
			header.Name += string(os.PathSeparator)
		}

		// Create writer for the file header and save content of the file.
		headerWriter, err := zipWriter.CreateHeader(header)
		if err != nil {
			return err
		}
		if info.IsDir() {
			return nil
		}
		f, err := os.Open(path)
		if err != nil {
			return err
		}
		defer f.Close()
		_, err = io.Copy(headerWriter, f)
		return err
	}
}
