package cmd

import (
	"bufio"
	"context"
	"fmt"
	"io"
	"os/exec"
	"path"

	"runtime"
	"strings"
	"time"
)

type NativeExe struct {
	Name               string
	SessionId          string
	CmdSuccFlag        string //启动成功标志, 如果不设置,
	CmdSuccFlag2       string
	CmdDurationForSucc int //如果没有启动标志，则启动多少时间后，没有报错，则判断启动成功
	CmdDurationForFail int //有启动标识，启动多少时间后，没有检测到成功标识判为启动失败

	ExeFile string   //执行文件
	Envs    []string //执行环境变量
	Params  []string //执行参数

	cancelRun CancelCmd

	MonitorQuit bool
	Quit        chan int

	Pid int
}

func (exe *NativeExe) Log(args ...interface{}) {
	fmt.Println(args...)
}

func (exe *NativeExe) StartForResult() (string, error) {
	exef := exe.ExeFile
	if runtime.GOOS == "windows" {
		if strings.LastIndex(exef, ".exe") == -1 {
			exef = exef + ".exe"
		}
	}
	cmd := exec.Command(exef, exe.Params...)
	ret, err := cmd.CombinedOutput()
	return string(ret), err
}

func (exe *NativeExe) Start() error {

	ctx, cancel := context.WithCancel(context.Background())

	exef := exe.ExeFile
	if runtime.GOOS == "windows" {
		if strings.LastIndex(exef, ".exe") == -1 {
			exef = exef + ".exe"
		}
	}

	cmd := exec.CommandContext(ctx, exef, exe.Params...)

	if len(exe.Envs) > 0 {
		cmd.Env = exe.Envs
	}
	canceled := false

	exe.Log(cmd.Dir, exef, exe.Params)

	exe.cancelRun = func() {
		if canceled {
			return
		}
		canceled = true
		cancel()
		exe.Log("cmd exe canceled")
	}

	stdoutIn, _ := cmd.StdoutPipe()
	stderrIn, _ := cmd.StderrPipe()

	reader := bufio.NewReader(stdoutIn)
	reader2 := bufio.NewReader(stderrIn)

	retChan := make(chan int)

	hasSuccFlag := len(exe.CmdSuccFlag) > 0

	go func() {
		checked := false
		for {
			line, err2 := reader.ReadString('\n')
			if err2 != nil || io.EOF == err2 {
				exe.Log("stdoutIn err=>", err2)
				retChan <- 1
				break
			}
			exe.Log(line)
			if !checked && hasSuccFlag && ((strings.LastIndex(line, exe.CmdSuccFlag) != -1) || (len(exe.CmdSuccFlag2) > 0 && strings.LastIndex(line, exe.CmdSuccFlag2) != -1)) {
				checked = true
				retChan <- 2
			}
		}
	}()

	go func() {
		checked := false
		for {
			line, err2 := reader2.ReadString('\n')
			if err2 != nil || io.EOF == err2 {
				exe.Log("stderrIn err=>", err2)
				retChan <- 1
				break
			}
			exe.Log(line)
			if !checked && hasSuccFlag && ((strings.LastIndex(line, exe.CmdSuccFlag) != -1) || (len(exe.CmdSuccFlag2) > 0 && strings.LastIndex(line, exe.CmdSuccFlag2) != -1)) {
				checked = true
				retChan <- 2
			}
		}
	}()
	exeBase := path.Base(exe.ExeFile)

	err := cmd.Start()
	if err != nil {
		exe.Log(exeBase, " start failed", err.Error())
		return err
	}
	waitSec := 0

	exe.Pid = cmd.Process.Pid

	for {
		conti := true
		waitSec += 1

		select {
		case ret := <-retChan:
			if ret == 1 {
				return fmt.Errorf("%s start failed", exeBase)
			}
			if ret == 2 { //启动成功
				conti = false
				time.Sleep(time.Second)
			}
		default:
			exe.Log("waiting ", exeBase, "to start ", waitSec, "/", exe.CmdDurationForFail)
			time.Sleep(1 * time.Second)
		}

		if !conti {
			break
		}

		if hasSuccFlag {
			if waitSec > exe.CmdDurationForFail {
				exe.cancelRun()
				e := fmt.Errorf("%s start failed", path.Base(exe.ExeFile))
				exe.Log(e)
				return e
			}
		} else {
			if waitSec > exe.CmdDurationForSucc { //启动成功
				break
			}
		}
	}

	exe.Log(path.Base(exe.ExeFile), "started")

	if exe.MonitorQuit {
		go func() {
			err := cmd.Wait()
			if err != nil {
				exe.Log("subsys ", exeBase, " quit error", err)
				exe.Quit <- 1
				return
			}
			exe.Quit <- 0
		}()
	}
	return nil
}

func (exe *NativeExe) Stop() {
	if exe.cancelRun != nil {
		exe.cancelRun()
		exe.cancelRun = nil
	}
}
